@@ -8,32 +8,28 @@ use warnings;
 use bytes;
 use Math::BigInt;
 use Socket;
+use constant syminfo => "DOCSIS::ConfigFile::Syminfo";
 
-our $ERROR     = '';
+our $ERROR     = q();
 our %SNMP_TYPE = (
-                    0x02 => [ 'INTEGER',        \&uint   ],
-                    0x04 => [ 'STRING',         \&string ],
-                    0x05 => [ 'NULLOBJ',        sub {}   ],
-                    0x40 => [ 'IPADDRESS',      \&ip     ],
-                    0x41 => [ 'COUNTER',        \&uint   ],
-                    0x42 => [ 'UNSIGNED',       \&uint   ],
-                    0x43 => [ 'TIMETICKS',      \&uint   ],
-                    0x44 => [ 'OPAQUE',         \&uint   ],
-                    0x46 => [ 'COUNTER64',      \&bigint ],
-                );
+    0x02 => [ 'INTEGER',    \&uint   ],
+    0x04 => [ 'STRING',     \&string ],
+    0x05 => [ 'NULLOBJ',    sub {}   ],
+    0x40 => [ 'IPADDRESS',  \&ip     ],
+    0x41 => [ 'COUNTER',    \&uint   ],
+    0x42 => [ 'UNSIGNED',   \&uint   ],
+    0x43 => [ 'TIMETICKS',  \&uint   ],
+    0x44 => [ 'OPAQUE',     \&uint   ],
+    0x46 => [ 'COUNTER64',  \&bigint ],
+);
 
 
-sub byte_size { #=============================================================
-    return $DOCSIS::ConfigFile::BYTE_SIZE{lc shift} || 0;
-}
-
 sub snmp_type { #=============================================================
     return $SNMP_TYPE{lc shift} || $SNMP_TYPE{4};
 }
 
 sub snmp_oid { #==============================================================
 
-    ### init
     my @input_oid   = @_;
     my @decoded_oid = (0);
     my $subid       = 0;
@@ -74,22 +70,20 @@ sub snmp_oid { #==============================================================
         $decoded_oid[1] -= 80;
     }
 
-    ### the end
     return join ".", @decoded_oid;
 }
 
 sub snmp_object { #===========================================================
 
-    ### init
     my $bin_string     = shift;
     my @data           = unpack "C*", $bin_string;
-    my $seq_id         = shift  @data;
-    my $message_length = shift  @data;
-    my $obj_id         = shift  @data;
-    my $oid_length     = shift  @data;
+    my $seq_id         = shift @data;
+    my $message_length = shift @data;
+    my $obj_id         = shift @data;
+    my $oid_length     = shift @data;
     my $oid            = snmp_oid(splice @data, 0, $oid_length);
-    my $value_type     = shift  @data;
-    my $value_length   = shift  @data;
+    my $value_type     = shift @data;
+    my $value_length   = shift @data;
     my $type           = snmp_type($value_type);
     my $bin_value      = substr $bin_string, 6 + $oid_length;
     my $value          = $type->[1]->($bin_value);
@@ -97,7 +91,6 @@ sub snmp_object { #===========================================================
     ### check value
     return {} unless(defined $value);
 
-    ### the end
     return {
         oid   => $oid,
         type  => $type->[0],
@@ -107,7 +100,6 @@ sub snmp_object { #===========================================================
 
 sub bigint { #================================================================
 
-    ### init
     my @bytes = unpack 'C*', shift;
     my $value = ($bytes[0] & 0x80) ? -1 : shift @bytes;
     my $int64 = Math::BigInt->new($value);
@@ -118,16 +110,14 @@ sub bigint { #================================================================
         $int64  = ($value << 8) | $_;
     }
 
-    ### the end
     return $int64;
 }
 
 sub uint { #==================================================================
 
-    ### init
     my @bytes  = unpack 'C*', shift;
     my $length = @bytes;
-    my $size   = byte_size('int');
+    my $size   = syminfo->byte_size('int');
     my $value  = ($bytes[0] & 0x80) ? -1 : 0;
 
     ### check
@@ -136,17 +126,15 @@ sub uint { #==================================================================
         return;
     }
 
-    ### the end
     $value = ($value << 8) | $_ for(@bytes);
     return $value;
 }
 
 sub ushort { #================================================================
 
-    ### init
     my $bin    = shift;
     my $length = length $bin;
-    my $size   = byte_size('short int');
+    my $size   = syminfo->byte_size('short int');
 
     ### check
     if($length > $size) {
@@ -154,44 +142,45 @@ sub ushort { #================================================================
         return;
     }
 
-    ### the end
     return unpack('n', $bin);
 }
 
-sub ushort_list { #===========================================================
-}
-
 sub uchar { #=================================================================
     return join "", unpack('C', shift);
 }
 
 sub vendorspec { #============================================================
 
-    ### init
-    my $bin    = shift;
-    my @input  = unpack "C*", $bin;
-    my $vendor = sprintf "%02x%02x%02x", @input[2..4];
-    my @data   = @input[5..$#input];
-    my @ret;
+    my $bin = shift;
+    my($vendor, @ret, $length);
 
-    while(@data) {
-        my $type   = shift  @data;
-        my $length = shift  @data;
-        my $value  = splice @data, 0, $length;
-        push @ret, {
-            type   => $type,
-            length => $length,
-            value  => $value,
-        };
+    $bin    =~ s/.(.)// or return; # remove the two first bytes
+    $length =  unpack "C*", $1;
+
+    ### find vendor
+    if($bin =~ s/(.{$length})//) {
+        my $f   = "%02x" x $length;
+        $vendor = sprintf "0x$f", unpack "C*", $1;
+    }
+
+    while($bin =~ s/^(.)(.)//) {
+        my $type   = unpack "C*", $1;
+        my $length = unpack "C*", $2;
+
+        if($bin =~ s/(.{$length})//) {
+            push @ret, {
+                type   => $type,
+                length => $length,
+                value  => hexstr($1),
+            };
+        }
     }
 
-    ### the end
     return $vendor, \@ret;
 }
 
 sub ip { #====================================================================
 
-    ### init
     my $bin     = shift;
     my $address = inet_ntoa($bin);
 
@@ -201,13 +190,11 @@ sub ip { #====================================================================
         return;
     }
 
-    ### the end
     return $address;
 }
 
 sub ether { #=================================================================
 
-    ### init
     my $bin    = shift;
     my $length = length $bin;
 
@@ -217,22 +204,16 @@ sub ether { #=================================================================
         return;
     }
 
-    ### the end
     return join ":", unpack("H2" x $length, $bin);
 }
 
-sub oid { #===================================================================
-    return snmp_oid(shift);
-}
-
 sub string { #================================================================
 
-    ### init
     my $bin = shift;
 
     ### hex string
     if($bin =~ /[\x00-\x1f\x7f-\xff]/) {
-        return "0x" .hexstr($bin);
+        return hexstr($bin);
     }
 
     ### normal string
@@ -241,12 +222,12 @@ sub string { #================================================================
     }
 }
 
-sub strzero { #===============================================================
-    return sprintf "%s", shift;
+sub hexstr { #================================================================
+    return "0x" .join("", unpack "H*", shift);
 }
 
-sub hexstr { #================================================================
-    return join "", unpack "H*", shift;
+sub mic { #===================================================================
+    return hexstr(@_);
 }
 
 #=============================================================================
@@ -263,10 +244,6 @@ See DOCSIS::ConfigFile
 
 =head1 FUNCTIONS
 
-=head2 byte_size(lc $arg)
-
-Returns the requested byte-size from DOCSIS::ConfigFile.
-
 =head2 snmp_type(lc $arg)
 
 Returns an array-ref to an array with two elements:
@@ -274,35 +251,70 @@ Returns an array-ref to an array with two elements:
  1) The string of the SNMP type.
  2) A reference to the function to decode the value.
 
-=head2 snmp_oid
+=head2 snmp_oid(@bytes)
 
-=head2 snmp_object
+Returns a numeric OID.
 
-=head2 bigint
+=head2 snmp_object($bytestring)
 
-=head2 uint
+Returns a hash-ref:
 
-Decodes a 8bit int.
+ {
+   oid   => "", # numeric OID
+   type  => "", # what kind of value (corresponding to C<snmp_type>)
+   value => "", # the oid value
+ }
 
-=head2 ushort
+=head2 bigint($bytestring)
 
-=head2 ushort_list
+Returns a C<Math::BigInt> object.
 
-=head2 uchar
+=head2 uint($bytestring)
 
-=head2 vendorspec
+Returns an unsigned integer: 0..2**32-1
 
-=head2 ip
+=head2 ushort($bytestring)
 
-=head2 ether
+Returns an unsigned short integer: 0..2**16-1
 
-=head2 oid
+=head2 uchar($bytesstring)
 
-=head2 string
+Returns an unsigned character: 0..2**8-1
 
-=head2 strzero
+=head2 vendorspec($bytestring)
 
-=head2 hexstr
+Returns a list containing ($vendor, \%nested).
+
+Example:
+
+  "0x001337", # vendors ID
+  {
+    type   => "24", # vendor specific type
+    value  => "42", # vendor specific value
+    length => "1",  # the length of the value meassured in bytes
+  },
+
+
+=head2 ip($bytestring)
+
+Returns an IPv4-address.
+
+=head2 ether($bytestring)
+
+Returns a MAC-address.
+
+=head2 string($bytestring)
+
+Returns human-readable string if it can, or the string hex-encoded if it
+cannot.
+
+=head2 hexstr($bytestring)
+
+Returns a value, printed as hex.
+
+=head2 mic($bytestring)
+
+Returns a value, printed as hex.
 
 =head1 AUTHOR
 
